@@ -2,23 +2,23 @@
 # pylint: disable=wrong-import-order, ungrouped-imports
 """Server module."""
 import os
-from collections import Counter
+import sys
 from urllib.parse import urlparse
 
-from buku import BukuDb
+from buku import BukuDb, __version__, network_handler
 from flask.cli import FlaskGroup
-from flask_api import status
+from flask_admin import Admin
+from flask_api import exceptions, FlaskAPI, status
 from flask_bootstrap import Bootstrap
 from flask_paginate import Pagination, get_page_parameter, get_per_page_parameter
 from markupsafe import Markup
-import arrow
 import click
 import flask
-from flask import (
+from flask import (  # type: ignore
+    __version__ as flask_version,
     abort,
     current_app,
     flash,
-    Flask,
     jsonify,
     redirect,
     render_template,
@@ -27,19 +27,21 @@ from flask import (
 )
 
 try:
-    from . import response, forms
+    from . import response, forms, views
 except ImportError:
-    from bukuserver import response, forms
+    from bukuserver import response, forms, views
 
 
-DEFAULT_PER_PAGE = 10
-DEFAULT_URL_RENDER_MODE = 'full'
 STATISTIC_DATA = None
 
+def get_bukudb():
+    """get bukudb instance"""
+    db_file = current_app.config.get('BUKUSERVER_DB_FILE', None)
+    return BukuDb(dbfile=db_file)
 
 def get_tags():
     """get tags."""
-    tags = getattr(flask.g, 'bukudb', BukuDb()).get_tag_all()
+    tags = getattr(flask.g, 'bukudb', get_bukudb()).get_tag_all()
     result = {
         'tags': tags[0]
     }
@@ -50,16 +52,60 @@ def get_tags():
     return res
 
 
+def network_handle_detail():
+    failed_resp = response.response_template['failure'], status.HTTP_400_BAD_REQUEST
+    url = request.data.get('url', None)
+    if not url:
+        return failed_resp
+    try:
+        res = network_handler(url)
+        return {'title': res[0], 'recognized mime': res[1], 'bad url': res[2]}
+    except Exception as e:
+        current_app.logger.debug(str(e))
+    return failed_resp
+
+
+def tag_list():
+    tags = get_bukudb().get_tag_all()
+    result = {'tags': tags[0]}
+    return result
+
+
+def tag_detail(tag):
+    bukudb = get_bukudb()
+    if request.method == 'GET':
+        tags = bukudb.get_tag_all()
+        if tag not in tags[1]:
+            raise exceptions.NotFound()
+        res = dict(name=tag, usage_count=tags[1][tag])
+    elif request.method == 'PUT':
+        res = None
+        try:
+            new_tags = request.data.get('tags').split(',')
+        except AttributeError as e:
+            raise exceptions.ParseError(detail=str(e))
+        result_flag = bukudb.replace_tag(tag, new_tags)
+        if result_flag:
+            res = response.response_template['success'], status.HTTP_200_OK
+        else:
+            res = response.response_template['failure'], status.HTTP_400_BAD_REQUEST
+    return res
+
+
 def update_tag(tag):
     res = None
     if request.method in ('PUT', 'POST'):
         new_tags = request.form.getlist('tags')
-        result_flag = getattr(flask.g, 'bukudb', BukuDb()).replace_tag(tag, new_tags)
+        result_flag = getattr(flask.g, 'bukudb', get_bukudb()).replace_tag(tag, new_tags)
         op_text = 'replace tag [{}] with [{}]'.format(tag, ', '.join(new_tags))
         if request.method == 'PUT' and result_flag and request.path.startswith('/api/'):
-            res = jsonify(response.response_template['success']), status.HTTP_200_OK, {'ContentType': 'application/json'}
+            res = (jsonify(response.response_template['success']),
+                   status.HTTP_200_OK,
+                   {'ContentType': 'application/json'})
         elif request.method == 'PUT' and request.path.startswith('/api/'):
-            res = jsonify(response.response_template['failure']), status.HTTP_400_BAD_REQUEST, {'ContentType': 'application/json'}
+            res = (jsonify(response.response_template['failure']),
+                   status.HTTP_400_BAD_REQUEST,
+                   {'ContentType': 'application/json'})
         elif request.method == 'POST' and result_flag:
             flash(Markup('Success {}'.format(op_text)), 'success')
             res = redirect(url_for('get_tags-html'))
@@ -71,24 +117,19 @@ def update_tag(tag):
     return res
 
 
-def chunks(l, n):
-    n = max(1, n)
-    return (l[i:i+n] for i in range(0, len(l), n))
-
-
 def bookmarks():
     """Bookmarks."""
     res = None
-    bukudb = getattr(flask.g, 'bukudb', BukuDb())
+    bukudb = getattr(flask.g, 'bukudb', get_bukudb())
     page = request.args.get(get_page_parameter(), type=int, default=1)
     per_page = request.args.get(
         get_per_page_parameter(),
         type=int,
         default=int(
-            current_app.config.get('BUKUSERVER_PER_PAGE', DEFAULT_PER_PAGE))
+            current_app.config.get('BUKUSERVER_PER_PAGE', views.DEFAULT_PER_PAGE))
     )
     url_render_mode = current_app.config['BUKUSERVER_URL_RENDER_MODE']
-    create_bookmarks_form = forms.CreateBookmarksForm()
+    create_bookmarks_form = forms.BookmarkForm()
     if request.method == 'GET':
         all_bookmarks = bukudb.get_rec_all()
         result = {
@@ -116,7 +157,7 @@ def bookmarks():
             current_app.logger.debug('total bookmarks:{}'.format(len(result['bookmarks'])))
             current_app.logger.debug('per page:{}'.format(per_page))
             pagination_total = len(result['bookmarks'])
-            bms = list(chunks(result['bookmarks'], per_page))
+            bms = list(views.chunks(result['bookmarks'], per_page))
             try:
                 result['bookmarks'] = bms[page-1]
             except IndexError as err:
@@ -175,11 +216,18 @@ def refresh_bookmarks():
     if request.method == 'POST':
         print(request.form['index'])
         print(request.form['threads'])
-        result_flag = getattr(flask.g, 'bukudb', BukuDb()).refreshdb(request.form['index'], request.form['threads'])
+        result_flag = getattr(
+            flask.g,
+            'bukudb',
+            get_bukudb()).refreshdb(request.form['index'], request.form['threads'])
         if result_flag:
-            res = jsonify(response.response_template['success']), status.HTTP_200_OK, {'ContentType': 'application/json'}
+            res = (jsonify(response.response_template['success']),
+                   status.HTTP_200_OK,
+                   {'ContentType': 'application/json'})
         else:
-            res = jsonify(response.response_template['failure']), status.HTTP_400_BAD_REQUEST, {'ContentType': 'application/json'}
+            res = (jsonify(response.response_template['failure']),
+                   status.HTTP_400_BAD_REQUEST,
+                   {'ContentType': 'application/json'})
     return res
 
 
@@ -190,8 +238,8 @@ def bookmark_api(id):
     except ValueError:
         return jsonify(response.response_template['failure']), status.HTTP_400_BAD_REQUEST, \
                {'ContentType': 'application/json'}
-    bukudb = getattr(flask.g, 'bukudb', BukuDb())
-    bookmark_form = forms.CreateBookmarksForm()
+    bukudb = getattr(flask.g, 'bukudb', get_bukudb())
+    bookmark_form = forms.BookmarkForm()
     is_html_post_request = request.method == 'POST' and not request.path.startswith('/api/')
     if request.method == 'GET':
         bookmark = bukudb.get_rec_by_id(id)
@@ -221,14 +269,26 @@ def bookmark_api(id):
     elif request.method == 'PUT' or is_html_post_request:
         if request.method == 'PUT':
             result_flag = bukudb.update_rec(
-                id, request.form['url'], request.form.get('title'), request.form['tags'], request.form['description'])
+                id,
+                request.form['url'],
+                request.form.get('title'),
+                request.form['tags'],
+                request.form['description'])
             if result_flag and not is_html_post_request:
-                res = jsonify(response.response_template['success']), status.HTTP_200_OK, {'ContentType': 'application/json'}
+                res = (jsonify(response.response_template['success']),
+                       status.HTTP_200_OK,
+                       {'ContentType': 'application/json'})
             elif not result_flag and not is_html_post_request:
-                res = jsonify(response.response_template['failure']), status.HTTP_400_BAD_REQUEST, {'ContentType': 'application/json'}
+                res = (jsonify(response.response_template['failure']),
+                       status.HTTP_400_BAD_REQUEST,
+                       {'ContentType': 'application/json'})
         elif is_html_post_request:
             result_flag = bukudb.update_rec(
-                id, bookmark_form.url.data, bookmark_form.title.data, bookmark_form.tags.data, bookmark_form.description.data)
+                id,
+                bookmark_form.url.data,
+                bookmark_form.title.data,
+                bookmark_form.tags.data,
+                bookmark_form.description.data)
             if result_flag:
                 flash(Markup('Success edit bookmark, id:{}'.format(id)), 'success')
             else:
@@ -239,9 +299,13 @@ def bookmark_api(id):
     else:
         result_flag = bukudb.delete_rec(id)
         if result_flag:
-            res = jsonify(response.response_template['success']), status.HTTP_200_OK, {'ContentType': 'application/json'}
+            res = (jsonify(response.response_template['success']),
+                   status.HTTP_200_OK,
+                   {'ContentType': 'application/json'})
         else:
-            res = jsonify(response.response_template['failure']), status.HTTP_400_BAD_REQUEST, {'ContentType': 'application/json'}
+            res = (jsonify(response.response_template['failure']),
+                   status.HTTP_400_BAD_REQUEST,
+                   {'ContentType': 'application/json'})
     return res
 
 
@@ -253,11 +317,16 @@ def refresh_bookmark(id):
                {'ContentType': 'application/json'}
     res = None
     if request.method == 'POST':
-        result_flag = getattr(flask.g, 'bukudb', BukuDb()).refreshdb(id, request.form['threads'])
+        result_flag = getattr(flask.g, 'bukudb',
+                              get_bukudb()).refreshdb(id, request.form['threads'])
         if result_flag:
-            res = jsonify(response.response_template['success']), status.HTTP_200_OK, {'ContentType': 'application/json'}
+            res = (jsonify(response.response_template['success']),
+                   status.HTTP_200_OK,
+                   {'ContentType': 'application/json'})
         else:
-            res = jsonify(response.response_template['failure']), status.HTTP_400_BAD_REQUEST, {'ContentType': 'application/json'}
+            res = (jsonify(response.response_template['failure']),
+                   status.HTTP_400_BAD_REQUEST,
+                   {'ContentType': 'application/json'})
     return res
 
 
@@ -269,14 +338,16 @@ def get_tiny_url(id):
                {'ContentType': 'application/json'}
     res = None
     if request.method == 'GET':
-        shortened_url = getattr(flask.g, 'bukudb', BukuDb()).tnyfy_url(id)
+        shortened_url = getattr(flask.g, 'bukudb', get_bukudb()).tnyfy_url(id)
         if shortened_url is not None:
             result = {
                 'url': shortened_url
             }
             res = jsonify(result)
         else:
-            res = jsonify(response.response_template['failure']), status.HTTP_400_BAD_REQUEST, {'ContentType': 'application/json'}
+            res = (jsonify(response.response_template['failure']),
+                   status.HTTP_400_BAD_REQUEST,
+                   {'ContentType': 'application/json'})
     return res
 
 
@@ -289,14 +360,16 @@ def get_long_url(id):
 
     res = None
     if request.method == 'GET':
-        bookmark = getattr(flask.g, 'bukudb', BukuDb()).get_rec_by_id(id)
+        bookmark = getattr(flask.g, 'bukudb', get_bukudb()).get_rec_by_id(id)
         if bookmark is not None:
             result = {
                 'url': bookmark[1],
             }
             res = jsonify(result)
         else:
-            res = jsonify(response.response_template['failure']), status.HTTP_400_BAD_REQUEST, {'ContentType': 'application/json'}
+            res = (jsonify(response.response_template['failure']),
+                   status.HTTP_400_BAD_REQUEST,
+                   {'ContentType': 'application/json'})
     return res
 
 
@@ -309,7 +382,7 @@ def bookmark_range_operations(starting_id, ending_id):
         return jsonify(response.response_template['failure']), status.HTTP_400_BAD_REQUEST, \
                {'ContentType': 'application/json'}
     res = None
-    bukudb = getattr(flask.g, 'bukudb', BukuDb())
+    bukudb = getattr(flask.g, 'bukudb', get_bukudb())
     max_id = bukudb.get_max_id()
     if starting_id > max_id or ending_id > max_id:
         return jsonify(response.response_template['failure']), status.HTTP_400_BAD_REQUEST, \
@@ -332,18 +405,26 @@ def bookmark_range_operations(starting_id, ending_id):
         for i in range(starting_id, ending_id + 1, 1):
             result_flag = bukudb.delete_rec(i)
             if result_flag is False:
-                return jsonify(response.response_template['failure']), status.HTTP_400_BAD_REQUEST, \
-                       {'ContentType': 'application/json'}
-        res = jsonify(response.response_template['success']), status.HTTP_200_OK, {'ContentType': 'application/json'}
+                return (jsonify(response.response_template['failure']),
+                        status.HTTP_400_BAD_REQUEST,
+                        {'ContentType': 'application/json'})
+        res = (jsonify(response.response_template['success']),
+               status.HTTP_200_OK,
+               {'ContentType': 'application/json'})
     elif request.method == 'PUT':
         for i in range(starting_id, ending_id + 1, 1):
             updated_bookmark = request.form[str(i)]
             result_flag = bukudb.update_rec(
-                i, updated_bookmark['url'], updated_bookmark['title'], updated_bookmark['tags'], updated_bookmark['description'])
+                i,
+                updated_bookmark['url'],
+                updated_bookmark['title'],
+                updated_bookmark['tags'],
+                updated_bookmark['description'])
 
             if result_flag is False:
-                return jsonify(response.response_template['failure']), status.HTTP_400_BAD_REQUEST, \
-                       {'ContentType': 'application/json'}
+                return (jsonify(response.response_template['failure']),
+                        status.HTTP_400_BAD_REQUEST,
+                        {'ContentType': 'application/json'})
         res = jsonify(response.response_template['success']), status.HTTP_200_OK, \
                {'ContentType': 'application/json'}
     return res
@@ -362,12 +443,12 @@ def search_bookmarks():
         all_keywords = False if all_keywords is None else all_keywords
         deep = False if deep is None else deep
         regex = False if regex is None else regex
-        all_keywords = \
-            all_keywords if type(all_keywords) == bool else all_keywords.lower() == 'true'
-        deep = \
-            deep if type(deep) == bool else deep.lower() == 'true'
-        regex = \
-            regex if type(regex) == bool else regex.lower() == 'true'
+        all_keywords = (
+            all_keywords if isinstance(all_keywords, bool) else
+            all_keywords.lower() == 'true'
+        )
+        deep = deep if isinstance(deep, bool) else deep.lower() == 'true'
+        regex = regex if isinstance(regex, bool) else regex.lower() == 'true'
     else:
         keywords = search_bookmarks_form.keywords.data
         all_keywords = search_bookmarks_form.all_keywords.data
@@ -375,7 +456,7 @@ def search_bookmarks():
         regex = search_bookmarks_form.regex.data
 
     result = {'bookmarks': []}
-    bukudb = getattr(flask.g, 'bukudb', BukuDb())
+    bukudb = getattr(flask.g, 'bukudb', get_bukudb())
     found_bookmarks = bukudb.searchdb(keywords, all_keywords, deep, regex)
     found_bookmarks = [] if found_bookmarks is None else found_bookmarks
     page = request.args.get(get_page_parameter(), type=int, default=1)
@@ -383,7 +464,7 @@ def search_bookmarks():
         get_per_page_parameter(),
         type=int,
         default=int(
-            current_app.config.get('BUKUSERVER_PER_PAGE', DEFAULT_PER_PAGE))
+            current_app.config.get('BUKUSERVER_PER_PAGE', views.DEFAULT_PER_PAGE))
     )
 
     res = None
@@ -403,7 +484,7 @@ def search_bookmarks():
             res = jsonify(result)
         else:
             pagination_total = len(result['bookmarks'])
-            bms = list(chunks(result['bookmarks'], per_page))
+            bms = list(views.chunks(result['bookmarks'], per_page))
             try:
                 result['bookmarks'] = bms[page-1]
             except IndexError as err:
@@ -418,118 +499,36 @@ def search_bookmarks():
                 'bukuserver/bookmarks.html',
                 result=result, pagination=pagination,
                 search_bookmarks_form=search_bookmarks_form,
-                create_bookmarks_form=forms.CreateBookmarksForm(),
+                create_bookmarks_form=forms.BookmarkForm(),
             )
     elif request.method == 'DELETE':
         if found_bookmarks is not None:
             for bookmark in found_bookmarks:
                 result_flag = bukudb.delete_rec(bookmark[0])
                 if result_flag is False:
-                    res = jsonify(response.response_template['failure']), status.HTTP_400_BAD_REQUEST, {'ContentType': 'application/json'}
+                    res = (jsonify(response.response_template['failure']),
+                           status.HTTP_400_BAD_REQUEST,
+                           {'ContentType': 'application/json'})
         if res is None:
-            res = jsonify(response.response_template['success']), status.HTTP_200_OK, {'ContentType': 'application/json'}
+            res = (jsonify(response.response_template['success']),
+                   status.HTTP_200_OK,
+                   {'ContentType': 'application/json'})
     return res
 
 
-def view_statistic():
-    bukudb = getattr(flask.g, 'bukudb', BukuDb())
-    global STATISTIC_DATA
-    statistic_data = STATISTIC_DATA
-    if not statistic_data or request.method == 'POST':
-        all_bookmarks = bukudb.get_rec_all()
-        netloc = [urlparse(x[1]).netloc for x in all_bookmarks]
-        tag_set = [x[3] for x in all_bookmarks]
-        tag_items = []
-        for tags in tag_set:
-            tag_items.extend([x.strip() for x in tags.split(',') if x.strip()])
-        tag_counter = Counter(tag_items)
-        title_items = [x[2] for x in all_bookmarks]
-        title_counter = Counter(title_items)
-        statistic_datetime = arrow.now()
-        STATISTIC_DATA = {
-            'datetime': statistic_datetime,
-            'netloc': netloc,
-            'tag_counter': tag_counter,
-            'title_counter': title_counter,
-        }
-    else:
-        netloc = statistic_data['netloc']
-        statistic_datetime = statistic_data['datetime']
-        tag_counter = statistic_data['tag_counter']
-        title_counter = statistic_data['title_counter']
-
-    netloc_counter = Counter(netloc)
-    unique_netloc_len = len(set(netloc))
-    colors = [
-        "#F7464A", "#46BFBD", "#FDB45C", "#FEDCBA",
-        "#ABCDEF", "#DDDDDD", "#ABCABC", "#4169E1",
-        "#C71585", "#FF4500", "#FEDCBA", "#46BFBD"]
-    show_netloc_table = False
-    if unique_netloc_len > len(colors):
-        max_netloc_item = len(colors)
-        netloc_colors = colors
-        show_netloc_table = True
-    else:
-        netloc_colors = colors[:unique_netloc_len]
-        max_netloc_item = unique_netloc_len
-    most_common_netlocs = netloc_counter.most_common(max_netloc_item)
-    most_common_netlocs = [
-        [val[0], val[1], netloc_colors[idx]] for idx, val in enumerate(most_common_netlocs)]
-
-    unique_tag_len = len(tag_counter)
-    show_tag_rank_table = False
-    if unique_tag_len > len(colors):
-        max_tag_item = len(colors)
-        tag_colors = colors
-        show_tag_rank_table = True
-    else:
-        tag_colors = colors[:unique_tag_len]
-        max_tag_item = unique_tag_len
-    most_common_tags = tag_counter.most_common(max_tag_item)
-    most_common_tags = [
-        [val[0], val[1], tag_colors[idx]] for idx, val in enumerate(most_common_tags)]
-
-    unique_title_len = len(title_counter)
-    show_title_rank_table = False
-    if unique_title_len > len(colors):
-        max_title_item = len(colors)
-        title_colors = colors
-        show_title_rank_table = True
-    else:
-        title_colors = colors[:unique_title_len]
-        max_title_item = unique_title_len
-    most_common_titles = title_counter.most_common(max_title_item)
-    most_common_titles = [
-        [val[0], val[1], title_colors[idx]] for idx, val in enumerate(most_common_titles)]
-
-    return render_template(
-        'bukuserver/statistic.html',
-        most_common_netlocs=most_common_netlocs,
-        netloc_counter=netloc_counter,
-        show_netloc_table=show_netloc_table,
-        most_common_tags=most_common_tags,
-        tag_counter=tag_counter,
-        show_tag_rank_table=show_tag_rank_table,
-        most_common_titles=most_common_titles,
-        title_counter=title_counter,
-        show_title_rank_table=show_title_rank_table,
-        datetime=statistic_datetime,
-        datetime_text=statistic_datetime.humanize(arrow.now(), granularity='second'),
-    )
-
-
 def create_app(config_filename=None):
     """create app."""
-    app = Flask(__name__)
-    per_page = int(os.getenv('BUKUSERVER_PER_PAGE', DEFAULT_PER_PAGE))
-    per_page = per_page if per_page > 0 else DEFAULT_PER_PAGE
+    app = FlaskAPI(__name__)
+    per_page = int(os.getenv('BUKUSERVER_PER_PAGE', str(views.DEFAULT_PER_PAGE)))
+    per_page = per_page if per_page > 0 else views.DEFAULT_PER_PAGE
     app.config['BUKUSERVER_PER_PAGE'] = per_page
-    url_render_mode = os.getenv('BUKUSERVER_URL_RENDER_MODE', DEFAULT_URL_RENDER_MODE)
+    url_render_mode = os.getenv('BUKUSERVER_URL_RENDER_MODE', views.DEFAULT_URL_RENDER_MODE)
     if url_render_mode not in ('full', 'netloc'):
-        url_render_mode = DEFAULT_URL_RENDER_MODE
+        url_render_mode = views.DEFAULT_URL_RENDER_MODE
     app.config['BUKUSERVER_URL_RENDER_MODE'] = url_render_mode
     app.config['SECRET_KEY'] = os.getenv('BUKUSERVER_SECRET_KEY') or os.urandom(24)
-    bukudb = BukuDb()
+    app.config['BUKUSERVER_DB_FILE'] = os.getenv('BUKUSERVER_DB_FILE')
+    bukudb = BukuDb(dbfile=app.config['BUKUSERVER_DB_FILE'])
     app.app_context().push()
     setattr(flask.g, 'bukudb', bukudb)
 
@@ -541,31 +540,78 @@ def create_app(config_filename=None):
     app.jinja_env.filters['netloc'] = lambda x: urlparse(x).netloc  # pylint: disable=no-member
 
     Bootstrap(app)
+    admin = Admin(
+        app, name='Buku Server', template_mode='bootstrap3',
+        index_view=views.CustomAdminIndexView(
+            template='bukuserver/home.html', url='/'
+        )
+    )
     # routing
-    app.add_url_rule('/api/tags', 'get_tags', get_tags, methods=['GET'])
-    app.add_url_rule('/tags', 'get_tags-html', get_tags, methods=['GET'])
-    app.add_url_rule('/api/tags/<tag>', 'update_tag', update_tag, methods=['PUT'])
-    app.add_url_rule('/tags/<tag>', 'update_tag-html', update_tag, methods=['POST'])
+    #  api
+    app.add_url_rule('/api/tags', 'get_tags', tag_list, methods=['GET'])
+    app.add_url_rule('/api/tags/<tag>', 'update_tag', tag_detail, methods=['GET', 'PUT'])
+    app.add_url_rule(
+        '/api/network_handle',
+        'networkk_handle',
+        network_handle_detail,
+        methods=['POST'])
     app.add_url_rule('/api/bookmarks', 'bookmarks', bookmarks, methods=['GET', 'POST', 'DELETE'])
-    app.add_url_rule('/bookmarks', 'bookmarks-html', bookmarks, methods=['GET', 'POST', 'DELETE'])
-    app.add_url_rule('/api/bookmarks/refresh', 'refresh_bookmarks', refresh_bookmarks, methods=['POST'])
-    app.add_url_rule('/api/bookmarks/<id>', 'bookmark_api', bookmark_api, methods=['GET', 'PUT', 'DELETE'])
-    app.add_url_rule('/bookmarks/<id>', 'bookmark_api-html', bookmark_api, methods=['GET', 'POST'])
-    app.add_url_rule('/api/bookmarks/<id>/refresh', 'refresh_bookmark', refresh_bookmark, methods=['POST'])
+    app.add_url_rule(
+        '/api/bookmarks/refresh',
+        'refresh_bookmarks',
+        refresh_bookmarks,
+        methods=['POST'])
+    app.add_url_rule(
+        '/api/bookmarks/<id>',
+        'bookmark_api',
+        bookmark_api,
+        methods=['GET', 'PUT', 'DELETE'])
+    app.add_url_rule(
+        '/api/bookmarks/<id>/refresh',
+        'refresh_bookmark',
+        refresh_bookmark,
+        methods=['POST'])
     app.add_url_rule('/api/bookmarks/<id>/tiny', 'get_tiny_url', get_tiny_url, methods=['GET'])
     app.add_url_rule('/api/bookmarks/<id>/long', 'get_long_url', get_long_url, methods=['GET'])
     app.add_url_rule(
         '/api/bookmarks/<starting_id>/<ending_id>',
         'bookmark_range_operations', bookmark_range_operations, methods=['GET', 'PUT', 'DELETE'])
-    app.add_url_rule('/api/bookmarks/search', 'search_bookmarks', search_bookmarks, methods=['GET', 'DELETE'])
-    app.add_url_rule('/bookmarks/search', 'search_bookmarks-html', search_bookmarks, methods=['GET'])
-    app.add_url_rule('/', 'index', lambda: render_template(
-        'bukuserver/index.html', search_bookmarks_form=forms.SearchBookmarksForm()))
-    app.add_url_rule('/statistic', 'statistic', view_statistic, methods=['GET', 'POST'])
+    app.add_url_rule(
+        '/api/bookmarks/search',
+        'search_bookmarks',
+        search_bookmarks,
+        methods=['GET', 'DELETE'])
+    #  non api
+    admin.add_view(views.BookmarkModelView(
+        bukudb, 'Bookmarks', page_size=per_page, url_render_mode=url_render_mode))
+    admin.add_view(views.TagModelView(
+        bukudb, 'Tags', page_size=per_page))
+    admin.add_view(views.StatisticView(
+        bukudb, 'Statistic', endpoint='statistic'))
     return app
 
 
-@click.group(cls=FlaskGroup, create_app=create_app)
+class CustomFlaskGroup(FlaskGroup):  # pylint: disable=too-few-public-methods
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        self.params[0].help = 'Show the program version'
+        self.params[0].callback = get_custom_version
+
+
+def get_custom_version(ctx, param, value):
+    if not value or ctx.resilient_parsing:
+        return
+    message = '%(app_name)s %(app_version)s\nFlask %(version)s\nPython %(python_version)s'
+    click.echo(message % {
+        'app_name': 'Buku',
+        'app_version': __version__,
+        'version': flask_version,
+        'python_version': sys.version,
+    }, color=ctx.color)
+    ctx.exit()
+
+
+@click.group(cls=CustomFlaskGroup, create_app=create_app)
 def cli():
     """This is a management script for the wiki application."""
 
